@@ -141,6 +141,20 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
                 extra_offset += extra_data_size
                 yield (extra_signature, extra_data)
 
+        def get_extra_value(extra, if_true, signature, exception_if_missing, min_length, exception_if_too_short):
+            if if_true:
+                try:
+                    value = extra[signature]
+                except KeyError:
+                    raise exception_if_missing()
+
+                if len(value) < min_length:
+                    raise exception_if_too_short()
+            else:
+                value = None
+
+            return value
+
         def weak_decrypt_decompress(chunks, decompress, is_done, num_unused):
             key_0 = 305419896
             key_1 = 591751049
@@ -176,8 +190,10 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
 
             return_unused(num_unused())
 
-        def aes_decrypt_decompress(chunks, decompress, is_done, num_unused):
-            aes_extra = extra[aes_extra_signature]
+        def aes_decrypt_decompress(chunks, decompress, is_done, num_unused, aes_extra):
+            if aes_extra[4] not in (1, 2, 3):
+                raise InvalidAESKeyLengthError(aes_extra[4])
+
             key_length = {1: 16, 2: 24, 3: 32}[aes_extra[4]]
             salt_length = {1: 8, 2: 12, 3: 16}[aes_extra[4]]
             compression = aes_extra[5:7]
@@ -241,12 +257,9 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
             if crc_32_actual != get_crc_32_expected():
                 raise CRC32IntegrityError()
 
-        version, flags, raw_compression, mod_time, mod_date, crc_32_expected, compressed_size, uncompressed_size, file_name_len, extra_field_len = \
+        version, flags, compression_raw, mod_time, mod_date, crc_32_expected, compressed_size, uncompressed_size, file_name_len, extra_field_len = \
             local_file_header_struct.unpack(get_num(local_file_header_struct.size))
 
-        file_name = get_num(file_name_len)
-        extra = dict(parse_extra(get_num(extra_field_len)))
-
         flag_bits = tuple(get_flag_bits(flags))
         if (
             flag_bits[4]      # Enhanced deflating
@@ -256,9 +269,13 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
         ):
             raise UnsupportedFlagsError(flag_bits)
 
-        is_weak_encrypted = flag_bits[0] and raw_compression != 99
-        is_aes_encrypted = flag_bits[0] and raw_compression == 99
-        is_aes_2_encrypted = is_aes_encrypted and extra[aes_extra_signature][0:2] == b'\x02\x00'
+        file_name = get_num(file_name_len)
+        extra = dict(parse_extra(get_num(extra_field_len)))
+
+        is_weak_encrypted = flag_bits[0] and compression_raw != 99
+        is_aes_encrypted = flag_bits[0] and compression_raw == 99
+        aes_extra = get_extra_value(extra, is_aes_encrypted, aes_extra_signature, MissingAESExtraError, 7, TruncatedAESExtraError)
+        is_aes_2_encrypted = is_aes_encrypted and aes_extra[0:2] == b'\x02\x00'
 
         if is_weak_encrypted and password is None:
             raise MissingZipCryptoPasswordError()
@@ -267,21 +284,22 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
             raise MissingAESPasswordError()
 
         compression = \
-            Struct('<H').unpack(extra[aes_extra_signature][5:7])[0] if is_aes_encrypted else \
-            raw_compression
+            Struct('<H').unpack(aes_extra[5:7])[0] if is_aes_encrypted else \
+            compression_raw
 
         if compression not in (0, 8):
             raise UnsupportedCompressionTypeError(compression)
 
-        has_data_descriptor = flag_bits[3]
-
         is_zip64 = compressed_size == zip64_compressed_size and uncompressed_size == zip64_compressed_size
-        uncompressed_size, compressed_size = \
-            Struct('<QQ').unpack(extra[zip64_size_signature]) if is_zip64 else \
+        zip64_extra = get_extra_value(extra, is_zip64, zip64_size_signature, MissingZip64ExtraError, 16, TruncatedZip64ExtraError)
+
+        has_data_descriptor = flag_bits[3]
+        uncompressed_size_raw, compressed_size = \
+            Struct('<QQ').unpack(zip64_extra) if is_zip64 else \
             (uncompressed_size, compressed_size)
         uncompressed_size = \
             None if has_data_descriptor and compression == 8 else \
-            uncompressed_size
+            uncompressed_size_raw
 
         decompressor = \
             get_dummy_decompressor(uncompressed_size) if compression == 0 else \
@@ -289,7 +307,7 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
 
         decompressed_bytes = \
             weak_decrypt_decompress(yield_all(), *decompressor) if is_weak_encrypted else \
-            aes_decrypt_decompress(yield_all(), *decompressor) if is_aes_encrypted else \
+            aes_decrypt_decompress(yield_all(), *decompressor, aes_extra) if is_aes_encrypted else \
             no_decrypt_decompress(yield_all(), *decompressor)
 
         get_crc_32_expected = \
@@ -342,6 +360,30 @@ class TruncatedDataError(DataError):
 class UnexpectedSignatureError(DataError):
     pass
 
+class MissingExtraError(DataError):
+    pass
+
+class MissingZip64ExtraError(MissingExtraError):
+    pass
+
+class MissingAESExtraError(MissingExtraError):
+    pass
+
+class TruncatedExtraError(DataError):
+    pass
+
+class TruncatedZip64ExtraError(TruncatedExtraError):
+    pass
+
+class TruncatedAESExtraError(TruncatedExtraError):
+    pass
+
+class InvalidExtraError(TruncatedExtraError):
+    pass
+
+class InvalidAESKeyLengthError(TruncatedExtraError):
+    pass
+
 class IntegrityError(DataError):
     pass
 
